<?php
namespace Zeedhi\PaymentPlatform\Implementations\PagarMe;

use Zeedhi\PaymentPlatform\InstantBuy\InstantBuyPlatform;
use Zeedhi\PaymentPlatform\InstantBuy\InstantBuyTransaction;

use Zeedhi\PaymentPlatform\NonInstantBuy\NonInstantBuyPlatform;
use Zeedhi\PaymentPlatform\NonInstantBuy\NonInstantBuyTransaction;

use Zeedhi\PaymentPlatform\CreditCard;
use Zeedhi\PaymentPlatform\Customer;
use Zeedhi\PaymentPlatform\Address;
use Zeedhi\PaymentPlatform\Item;
use Zeedhi\PaymentPlatform\RequestURL;
use Zeedhi\PaymentPlatform\Exception;

class PaymentImpl implements InstantBuyPlatform, NonInstantBuyPlatform{

    const ENDPOINT = "https://api.pagar.me";

    const POST = "POST";


    /** @var String */
    private $apiKey;

    /** @var array */
    private $requestHeaders;

    /** @var HttpRequest */
    private $httpRequest;


    public function __construct($apiKey, HttpRequest $httpRequest){
        $this->apiKey = $apiKey;
        $this->httpRequest = $httpRequest;
        $this->requestHeaders = array(
            'Content-Type:application/json'
        );
    }

    public function createToken(CreditCard $card, Customer $customer) {
        $request = array(
            "api_key" => $this->apiKey,
            "card_number" => $card->getNumbers(),
            "card_cvv" => $card->getCvv(),
            "card_expiration_date" => $card->getExpDateMonth().$card->getExpDateYear(),
            "card_holder_name" => $card->getHolderName()
        );
        $responseData = $this->createRequest(RequestURL::PAGARME_CARDS, self::POST, $request);
        return array(
            'InstantBuyKey' => $responseData['id']
        );
    }

    public function formatCustomer($customer){
        $customerArray['external_id'] = $customer->getId();
        $customerArray['name'] = $customer->getName();
        $customerArray['type'] = 'individual';
        $customerArray['country'] = $customer->getAddress()->getCountry();
        $customerArray['email'] = $customer->getEmail();
        $customerArray['documents'] = array(
            array(
                'type' => 'cpf',
            'number' => $customer->getDocument()
            )
        );
        $customerArray['phone_numbers'] = array(
            $customer->getPhone()
        );
        
        return $customerArray;
    }

    public function formatBilling($customer){
        $billingArray['name'] = $customer->getName();
        $billingArray['address'] = array(
            'country' => $customer->getAddress()->getCountry(),
            'state' => $customer->getAddress()->getState(),
            'city' => $customer->getAddress()->getCity(),
            'street' => $customer->getAddress()->getStreet(),
            'zipcode' => $customer->getAddress()->getZipCode()
        );
        
        return $billingArray;
    }

    public function formatItems($items){
        $itemsArray = array();
        foreach($items as $item){
            $currentItem['id'] = $item->getId();
            $currentItem['title'] = $item->getTitle();
            $currentItem['unit_price'] = $item->getUnitPrice();
            $currentItem['quantity'] = $item->getQuantity();
            $currentItem['tangible'] = false;

            array_push($itemsArray, $currentItem);
        }
        return $itemsArray;
    }

    public function instantBuyTransaction(InstantBuyTransaction $instantBuyTransaction) {
        $request = array(
            "api_key" => $this->apiKey,
            "capture" => false,
            "amount" => $instantBuyTransaction->getValue(),
            "card_id" => $instantBuyTransaction->getCreditCardToken(),
            "customer" => $this->formatCustomer($instantBuyTransaction->getCustomer()),
            "billing" => $this->formatBilling($instantBuyTransaction->getCustomer()),
            "items" => $this->formatItems($instantBuyTransaction->getItems())
        );
        $responseData = $this->createRequest(RequestURL::PAGARME_TRANSACTIONS, self::POST, $request);
        $captureUrl = RequestURL::PAGARME_TRANSACTIONS.'/'.$responseData['id'].RequestURL::PAGARME_CAPTURE;
        return array(
            'status' => $responseData['status'],
            'message' => $responseData['refuse_reason'],
            'captureUrl' => $captureUrl,
            'paymentId' => $responseData['id'],
            'nsu' => isset($responseData['nsu']) ? $responseData['nsu'] : null
        );
    }
    
    public function nonInstantBuyTransaction(NonInstantBuyTransaction $nonInstantBuyTransaction) {
        $request = array(
            "api_key" => $this->apiKey,
            "capture" => false,
            "amount" => $nonInstantBuyTransaction->getValue(),
            "card_number" => $nonInstantBuyTransaction->getCreditCard()->getNumbers(),
            "card_cvv" => $nonInstantBuyTransaction->getCreditCard()->getCvv(),
            "card_expiration_date" => $nonInstantBuyTransaction->getCreditCard()->getExpDateMonth().$nonInstantBuyTransaction->getCreditCard()->getExpDateYear(),
            "card_holder_name" => $nonInstantBuyTransaction->getCreditCard()->getHolderName(),
            "customer" => $this->formatCustomer($nonInstantBuyTransaction->getCustomer()),
            "billing" => $this->formatBilling($nonInstantBuyTransaction->getCustomer()),
            "items" => $this->formatItems($nonInstantBuyTransaction->getItems())
        );
        $responseData = $this->createRequest(RequestURL::PAGARME_TRANSACTIONS, self::POST, $request);
        $captureUrl = RequestURL::PAGARME_TRANSACTIONS.'/'.$responseData['id'].RequestURL::PAGARME_CAPTURE;
        return array(
            'status' => $responseData['status'],
            'message' => $responseData['refuse_reason'],
            'captureUrl' => $captureUrl,
            'paymentId' => $responseData['id'],
            'nsu' => isset($responseData['nsu']) ? $responseData['nsu'] : null
        );
    }

    public function captureTransaction($url, $paymentId) {
        $request = array(
            'api_key' => $this->apiKey
        );
        $captureResponse = $this->createRequest($url, self::POST, $request);
        return array(
            'status' => $captureResponse['status'],
            'message' => $captureResponse['refuse_reason'],
            'paymentId' => $captureResponse['id'],
            'nsu' => isset($captureResponse['nsu']) ? $captureResponse['nsu'] : null
        );
    }

    private function createRequest($url, $verb, $fields){
        $this->configCurl($url);
        $this->httpRequest->setOption(CURLOPT_CUSTOMREQUEST, $verb);
        $this->httpRequest->setOption(CURLOPT_POSTFIELDS, json_encode($fields));
        return $this->responseTreatment();
    }

    private function configCurl($url) {
        $this->httpRequest->setUrl(self::ENDPOINT.$url);
        $this->httpRequest->setOption(CURLOPT_HTTPHEADER, $this->requestHeaders);
        $this->httpRequest->setOption(CURLOPT_RETURNTRANSFER, true);
    }

    private function responseTreatment() {
        $response = json_decode($this->httpRequest->execute(), true);
        $httpStatusCode = $this->httpRequest->getInfo(CURLINFO_HTTP_CODE);
        $this->httpRequest->close();
        if($httpStatusCode == '200') {
            return $response;
        } else {
            throw Exception::apiPagarMe($httpStatusCode, $response['errors'][0]['message']);
        }
    }

    public static function factory($apiKey) {
        return new self($apiKey, new HttpRequest);
    }

}