describe("ZeedhiFilters", function () {
	var $filter;
	var oldWindowLang = window.lang;

	var i18nFilter;
	var columnFormatFilter;
	var uniqueFilter;
	var oderByHeaderFilter;
	var hasWidgetToExpandFilter;
	var startsWithFilter;
	var isLikeFilter;
	var rangeFilter;
	var containsFilter;
	var paginateFilter;
	var _ScreenService;

	beforeEach(function () {
		window.lang = "pt_br";
		// Load zeedhi modules to use the filters from ZeedhiFilters module
		module("ZeedhiApp");
		module("ZeedhiServices");
		module("ZeedhiFilters");

		module(function($provide) {
			$provide.service('ScreenService', function() {
				var words = {
					'pt_br': {
						'I want to sing': 'Eu quero cantar',
						'Testing is awesome': 'Testar é top!'
					},
					'en_us': {
						'Nome': 'Name'
					},
					'es': {
						'what is the answer to life the universe and everything?': '42'
					},
					'default': {
						'Wow its a phrase': 'Testing default'
					}
				};

				var language;
				this.setLanguage = function(_language) {
					language = _language;
					return Q.when(words[language] || {});
				};

				this.i18n = function(word) {
					return words[language] && words[language][word] || words.default && words.default[word] || word;
				};
			});
		});

		inject(function (_$filter_, ScreenService) {
			_ScreenService = ScreenService;
			$filter = _$filter_;
			i18nFilter = $filter("i18n");
			columnFormatFilter = $filter("columnFormat");
			uniqueFilter = $filter("unique");
			oderByHeaderFilter = $filter("orderByHeader");
			hasWidgetToExpandFilter = $filter("hasWidgetToExpand");
			startsWithFilter = $filter("startsWith");
			isLikeFilter = $filter("isLike");
			rangeFilter = $filter("range");
			containsFilter = $filter("contains");
			paginateFilter = $filter("paginate");
		});
	});

	afterEach(function () {
		window.lang = oldWindowLang;
	});

	it("i18n: Should return the argument passed to the filter if it was not found in words", function () {
		expect(i18nFilter("someWord")).toEqual("someWord");
	});

	it("i18n: Should translate from the passed language", function (done) {
		_ScreenService.setLanguage('pt_br').then(function() {
			expect(i18nFilter("Testing is awesome")).toEqual("Testar é top!");
		}).then(function() {
			return _ScreenService.setLanguage('en_us');
		}).then(function() {
			expect(i18nFilter("Nome")).toEqual("Name");
		}).then(function() {
			return _ScreenService.setLanguage('es');
		}).then(function() {
			expect(i18nFilter("what is the answer to life the universe and everything?", "es")).toEqual("42");
			done();
		});
	});

	it("i18n: Should fallback to the words default if language is not found in words", function () {
		expect(i18nFilter("Wow its a phrase", 'null_language')).toEqual("Testing default");
	});

	it("columnFormat: should call the maskEngine method and work for built in zeedhi mask methods", function () {
		expect(columnFormatFilter("2.0", {mask: {type: 'number'}})).toEqual('2');
		expect(columnFormatFilter("2.0", {mask: {type: 'float'}})).toEqual('2,00');
		expect(columnFormatFilter("2.0", {mask: {type: 'currency'}})).toEqual('R$2,00');
		// TODO: Please check the maskEngine implementation.
		// Date by default returns a value with hour and datetime by default returns a value without hour
		// expect(columnFormatFilter("01/02/2016", {mask: {type: 'date'}})).toEqual('02/01/2016 00:00:00');
		expect(columnFormatFilter("01/02/2016", {mask: {type: 'datetime'}})).toEqual('01/02/2016');
		expect(columnFormatFilter("aaa", {mask: {type: 'fix'}})).toEqual('aaa');
		expect(columnFormatFilter("1", {mask: {type: 'zerofill'}})).toEqual('0001');

		// TODO: test metric mask
	});

	it("columnFormat: Should work passing params", function () {
		var params = {
			thousands: ',',
			decimal: ',',
			precision: 3,
			symbol: 'R$'
		};

		expect(columnFormatFilter("2.550", {mask: {type: 'number', params: params}})).toEqual('3');
		expect(columnFormatFilter("2.550", {mask: {type: 'float', params: params}})).toEqual('2,550');
		expect(columnFormatFilter("2.550", {mask: {type: 'currency', params: params}})).toEqual('R$2,550');
		// TODO: make tests to other formats with param
	});

	it("unique: Should return the array when the parameter filterOn false", function () {
		var array = [1,2,3];
		expect(uniqueFilter(array, false)).toEqual(array);
	});

	it("unique: Should return the array when the parameter filterOn is false", function () {
		var array = [1,2,3];
		expect(uniqueFilter(array)).toEqual(array);
	});

	it("unique: Should filter unique values comparing the objects by a provided key", function () {
		var array = [{name: "Wagner"}, {name: "Wagner"}, {name: "Uabafet"}];
		var newItems = uniqueFilter(array, "name");
		expect(newItems.length).toBe(2);
	});

	it("unique: Should filter unique values when passing no key to compare", function () {
		var array = ["Wagner", "Foo", "Bar", "Foo"];
		var newItems = uniqueFilter(array);
		expect(newItems.length).toBe(3);
	});

	it("orderByHeader: Should return the passed array when not passing an orderBy property", function () {
		var array = ["Wagner", "Foo", "Bar", "Foo"];
		expect(oderByHeaderFilter(array)).toEqual(array);
	});

	it("orderByHeader: Should order the values by a provided key", function () {
		var array = [{name: "Wagnerz"}, {name: "Wagner"}, {name: "Uabafet"}];
		expect(oderByHeaderFilter(array, "name")).toEqual([{name: "Uabafet"}, {name: "Wagner"}, {name: "Wagnerz"}]);
	});

	it("orderByHeader: Should reverse the order if passing true to the reverse parameter", function () {
		var array = [{name: "Wagner"}, {name: "Wagnerz"}, {name: "Uabafet"}];
		expect(oderByHeaderFilter(array, "name", true)).toEqual([{name: "Wagnerz"}, {name: "Wagner"}, {name: "Uabafet"}]);
	});

	it("reverse: Should reverse the array", function () {
		expect($filter("reverse")([1,2,3,4])).toEqual([4,3,2,1]);
	});

	it("hasWidgetToExpand: should return the widgets that will expand or the widgets", function () {
		var widgets = [{ expand: true }, {expand: false}];
		expect(hasWidgetToExpandFilter(widgets).length).toBe(1);
		widgets = [{ expand: false }, {expand: false}];
		expect(hasWidgetToExpandFilter(widgets)).toEqual(widgets);
	});

	it("startsWith: should return all the items that starts with the query", function () {
		var items = [{bla: "starting with"}, {bla: "not starting with"}];
		expect(startsWithFilter(items, "bla", "starting")).toEqual([items[0]]);
		expect(startsWithFilter(false)).toBe(false);
	});

	it("isLike: should return all the items that satisfy the query", function () {
		var items = [{bla: "starting with"}, {bla: "not starting with"}];
		expect(isLikeFilter(items, "bla", "with")).toEqual(items);
		expect(isLikeFilter(false)).toBe(false);
	});

	it("range: should get an array of a range of numbers", function () {
		expect(rangeFilter("anything, the input is not important", 0, 5)).toEqual([0, 1, 2, 3, 4, 5]);
	});

	it("isLike: should return all the items that satisfy the query", function () {
		var items = [{bla: "starting with"}, {bla: "not starting with"}];
		expect(isLikeFilter(items, "bla", "with")).toEqual(items);
	});

	it("containers: should get a match for the items with an regex", function () {
		var items = [{bla: "WOOOOW icrazyss"}, {bla: "not starting with"}];
		expect(containsFilter(items, "bla", "crazy")).toEqual([items[0]]);
	});

	it("containers: should return the items if there is no query", function () {
		var items = [{bla: "WOOOOW icrazyss"}, {bla: "not starting with"}];
		expect(containsFilter(items, "bla")).toEqual(items);
	});

	it("paginate: should return an empty array if there is no input", function () {
		expect(paginateFilter()).toEqual([]);
	});

	it("paginate: should paginate some items from start to end", function () {
		expect(paginateFilter([1, 2, 3, 4, 5], 0, 3)).toEqual([1, 2, 3]);
	});
});